@@ -2,37 +2,53 @@
 # hdl21.module 
 The `Module` module (get it?)
 
-This module primarily defines the core circuit-reuse element `hdl21.Module`. 
-It also includes the fun machinery for class-syntax creation of `Module`s, 
-particularly the `@module` (lower-case) decorator-function. 
+This module primarily defines: 
+* The core circuit-reuse element `hdl21.Module`. 
+* The `@module` (lower-case) decorator-function, for class-syntax creation of `Module`s
 """
 
 import inspect
-from textwrap import dedent
 from types import ModuleType
-from typing import Any, Optional, List, Union, get_args, Tuple, Type
+from typing import Any, Optional, List, Union, get_args, Type
 from pydantic.dataclasses import dataclass
 from dataclasses import field
 
 # Local imports
+from .params import NoParams, HasNoParams
 from .signal import Signal, Visibility
 from .instance import calls_instantiate, Instance, InstArray
-from .interface import InterfaceInstance
+from .bundle import BundleInstance
 
 # Type-alias for HDL objects storable as `Module` attributes
-ModuleAttr = Union[Signal, Instance, InstArray, InterfaceInstance]
-
-
-def _is_module_attr(val: Any) -> bool:
-    """ Boolean indication of whether `val` is a valid `hdl21.Module` attribute. """
-    return isinstance(val, get_args(ModuleAttr))
+ModuleAttr = Union[Signal, Instance, InstArray, BundleInstance]
 
 
 @calls_instantiate
 class Module:
-    """
-    # Module
+    """ # Module
+    
     The central element of hardware re-use.
+    Hdl21 `Module`s are static combinations of HDL objects such as `Signal`s, `Instance`s of other `Module`s, and `Bundle`s. 
+
+    Two primary methods (directly) create `Module`s. The `Module` (capital M) class-constructor accepts a single optional `name` field. 
+    This serves as the name of an initially-empty Module-definition. Attributes can then be added programmatically. 
+    ```
+    m = Module(name="MyModule")
+    m.inp = h.Input(width=5)
+    m.add(h.Output(name="out", width=4))
+    ```
+
+    The `hdl21.module` (lower-case m) decorator-function is a convenient way to create a `Module` from a Python class-definition.
+    Each attribute in the class-body is converted to a module attribute, and the class name is used as the module's name.
+    ```
+    @h.module 
+    class MyModule:
+        inp = h.Input(width=5)
+        out = h.Output(width=4)
+    ```
+
+    Hdl21 Modules *do not* contain behavior in the sense of procedural HDLs. Nor do they contain parameters. 
+    Parametric hardware is produced through Hdl21's `generator` facility, which defines python functions which create and return `Module`s. 
     """
 
     def __init__(self, *, name: Optional[str] = None):
@@ -41,7 +57,7 @@ class Module:
         self.signals = dict()
         self.instances = dict()
         self.instarrays = dict()
-        self.interfaces = dict()
+        self.bundles = dict()
         self.namespace = dict()  # Combination of all these
 
         self._pymodule = _caller_pymodule()  #  (Python) module where called
@@ -63,21 +79,20 @@ class Module:
             "signals",
             "instances",
             "instarrays",
-            "interfaces",
+            "bundles",
             "namespace",
             "add",
             "get",
         ]
         if key in banned:
-            raise RuntimeError(
-                f"Error attempting to over-write protected attribute {key} of Module {self}"
-            )
+            msg = f"Error attempting to over-write protected attribute {key} of Module {self}"
+            raise RuntimeError(msg)
         # Special case(s)
         if key == "name":
             return super().__setattr__(key, val)
 
-        if not _is_module_attr(val):
-            raise TypeError(f"Invalid Module attribute {val} for {self}")
+        # Check it's a valid attribute-type
+        _assert_module_attr(self, val)
 
         # Checks out! Name `val` and add it to our type-based containers.
         val.name = key
@@ -85,32 +100,36 @@ class Module:
         return None
 
     def add(self, val: ModuleAttr, *, name: Optional[str] = None) -> ModuleAttr:
-        """ Add a named HDL object into one of our internal dictionaries.
+        """ Add a Module attribute.
 
-        This allows for programmatic insertion of attributes whose names are not legal Python identifiers,
+        `Module.add` allows for programmatic insertion of attributes whose names are not legal Python identifiers,
         such as keywords ('in', 'from') and those including invalid characters.
 
         The added object `val` is also provided as the return value, enabling chaining-style usages such as
         ```python
         instance.inp = MyModule.add(h.Input(name="in", width=5))
         ``` 
-        and similar. """
+        and similar. 
 
-        if not _is_module_attr(val):
-            raise TypeError(f"Invalid Module attribute {val} for {self}")
+        Attribute naming comes from one of either: 
+        * `val`'s `name` attribute, if it has one, or 
+        * The optional `name` argument.
+        Only one of the two name-sources is allowed per call. 
+        """
+
+        # Check it's a valid attribute-type
+        _assert_module_attr(self, val)
 
         # Now sort out naming. We get two name-sources:
         # (a) the function-argument `name` and (b) the value's `name` attribute.
         # One or the other (and not both) must be set.
-        if name is None and val.name is None:
-            raise RuntimeError(
-                f"Invalid anonymous attribute {val} cannot be added to Module {self.name}"
-            )
-        if name is not None and val.name is not None:
-            raise RuntimeError(
-                f"Conflicting names {name} and {val.name} cannot be added to Module {self.name}"
-            )
-        if name is not None:
+        if name is None and val.name is None:  # Neither set, fail.
+            msg = f"Anonymous attribute {val} cannot be added to Module {self.name}"
+            raise RuntimeError(msg)
+        if name is not None and val.name is not None:  # Both set, fail.
+            msg = f"{val} with conflicting names {name} and {val.name} cannot be added to Module {self.name}"
+            raise RuntimeError(msg)
+        if name is not None:  # One or the other set - great.
             val.name = name
 
         # Now `val.name` is set appropriately.
@@ -118,8 +137,8 @@ class Module:
         return self._add(val)
 
     def _add(self, val: ModuleAttr) -> ModuleAttr:
-        """ Internal `add` and `setattr` implementation. 
-        Primarily sort `val` into one of our type-based containers. """
+        """ Internal `add` and `setattr` implementation. Primarily sort `val` into one of our type-based containers. 
+        Layers above `_add` must ensure that `val` has its `name` attribute before calling this method. """
 
         if isinstance(val, Signal):
             self.namespace[val.name] = val
@@ -133,14 +152,30 @@ class Module:
         elif isinstance(val, InstArray):
             self.instarrays[val.name] = val
             self.namespace[val.name] = val
-        elif isinstance(val, InterfaceInstance):
-            self.interfaces[val.name] = val
+        elif isinstance(val, BundleInstance):
+            self.bundles[val.name] = val
             self.namespace[val.name] = val
         else:
-            raise TypeError(f"Invalid Module attribute {val} for {self}")
+            # The next line *should* never be reached, as outer layers should have checked `_is_module_attr`.
+            # Nonetheless gotta raise an error if we get here, somehow.
+            self._attr_type_error(val)
 
-        # Give it a reference to us
+        # Give it a reference to us.
+        #
+        # ## Note:
+        # Parent-testing *can* be done here instead of at elaboration time.
+        # It's not clear that this would be a good idea though.
+        # Attributes which are *copied* (for example) keep the `_parent_module` member, which is often helpful to just over-write.
+        # Nonetheless if "setattr-time" failure is desired, this is where it will go:
+        #
+        # _parent = getattr(val, "_parent_module", None) # Note many attributes will not have this member, until this assignment.
+        # if _parent is not None and _parent is not self:
+        #     msg = f"{val.name} being added to {self} already has a parent-module {val._parent_module}"
+        #     raise RuntimeError(msg)
+        #
+        # Ok, now actually give it a reference to us:
         val._parent_module = self
+
         # And return our newly-added attribute
         return val
 
@@ -162,13 +197,8 @@ class Module:
 
     def __init_subclass__(cls, *_, **__):
         """ Sub-Classing Disable-ization """
-        raise RuntimeError(
-            dedent(
-                f"""\
-                Error attempting to create {cls.__name__}
-                Sub-Typing hdl21.Module is not supported. """
-            )
-        )
+        msg = f"Error attempting to create {cls.__name__}. Sub-Typing {cls} is not supported."
+        raise RuntimeError(msg)
 
     def __repr__(self) -> str:
         if self.name:
@@ -176,14 +206,14 @@ class Module:
         return f"Module(_anon_)"
 
     @property
-    def interface_ports(self) -> dict:
-        """ Port-Exposed Interface Instances """
-        return {name: intf for name, intf in self.interfaces.items() if intf.port}
+    def bundle_ports(self) -> dict:
+        """ Port-Exposed Bundle Instances """
+        return {name: bundle for name, bundle in self.bundles.items() if bundle.port}
 
     @property
     def io(self) -> dict:
-        """ Combined Dictionary of `Signal`-valued and `Interface`-valued ports """
-        rv = self.interface_ports
+        """ Combined Dictionary of `Signal`-valued and `Bundle`-valued ports """
+        rv = self.bundle_ports
         rv.update(self.ports)
         return rv
 
@@ -220,24 +250,6 @@ class Module:
             raise RuntimeError(f"Cannot invoke pickling on unnamed Module {self}")
         return self._qualname()
 
-    @property
-    def _data(self) -> "_ModuleData":
-        """ Retrieve a `_ModuleData` corresponding to our namespace """
-        if self._updated or self._moduledata is None:
-            ns = self.__getattribute__("namespace")
-            self._moduledata = _ModuleData(name=self.name, namespace=list(ns.values()))
-        return self._moduledata
-
-
-@dataclass(frozen=True)
-class _ModuleData:
-    """ Immutable data-class representation of `Module` content. 
-    Designed for internal use by `Module`, particularly for sake of 
-    hashing, equality testing, and serialization. """
-
-    qualname: str
-    namespace: Tuple  # More specifically Tuple[ModuleAttr], but specifying so triggers a pydantic bug
-
 
 def module(cls: type) -> Module:
     """
@@ -263,7 +275,7 @@ def module(cls: type) -> Module:
     `hdl21.Modules` are strongly-typed containers of other hardware objects:
     * `Signals` and `Ports`
     * Instances of other `Modules`
-    * Structured connections via `Interfaces`
+    * Structured connections via `Bundles`
 
     Attempts to set module attributes of any other types will raise a `TypeError`.
     Notably this includes any behavioral elements implemented by Python functions.
@@ -284,7 +296,7 @@ def module(cls: type) -> Module:
     dunders = dict()
     unders = dict()
 
-    # Take a lap through the class dictionary, type-check everything and assign relevant attributes to the interface
+    # Take a lap through the class dictionary, type-check everything and assign relevant attributes to the bundle
     for key, val in cls.__dict__.items():
         if key.startswith("__"):
             dunders[key] = val
@@ -311,11 +323,11 @@ class ExternalModule:
     Parameter-values are checked to be instances of `paramtype` at creation time. 
     """
 
-    name: str
-    port_list: List[Signal]
-    paramtype: Type = object
+    name: str  # Module name. Used *directly* when exporting.
+    port_list: List[Signal]  # Ordered Ports
+    paramtype: Type = HasNoParams  # Parameter-type `paramclass`
     desc: Optional[str] = None  # Description
-    domain: Optional[str] = None
+    domain: Optional[str] = None  # Domain name, for references upon export
     pymodule: Optional[ModuleType] = field(repr=False, init=False, default=None)
     importpath: Optional[List[str]] = field(repr=False, init=False, default=None)
 
@@ -330,11 +342,10 @@ class ExternalModule:
             if not p.name:
                 raise ValueError(f"Unnamed Primitive Port {p} for {self.name}")
             if p.vis != Visibility.PORT:
-                raise ValueError(
-                    f"Invalid Primitive Port {p.name} on {self.name}; must have PORT visibility"
-                )
+                msg = f"Invalid Primitive Port {p.name} on {self.name}; must have PORT visibility"
+                raise ValueError(msg)
 
-    def __call__(self, params) -> "ExternalModuleCall":
+    def __call__(self, params: Any = NoParams) -> "ExternalModuleCall":
         return ExternalModuleCall(module=self, params=params)
 
     @property
@@ -388,18 +399,64 @@ class ExternalModuleCall:
     def __post_init_post_parse__(self):
         # Type-validate our parameters
         if not isinstance(self.params, self.module.paramtype):
-            raise TypeError(
-                f"Invalid parameters {self.params} for ExternalModule {self.module}. Must be {self.module.paramtype}"
-            )
+            msg = f"Invalid parameters {self.params} for ExternalModule {self.module}. Must be {self.module.paramtype}"
+            raise TypeError(msg)
 
     @property
     def ports(self) -> dict:
         return self.module.ports
 
 
-def _caller_pymodule():
-    """ Find the first frame not from *this* file
-    Sometimes this will be a Generator. That's OK, they'll figure it out. """
+""" 
+(Python) Module-Level Functions
+
+Could be methods, but are generally kept here to prevent expanding the `Module` namespace.
+"""
+
+
+def _is_module_attr(val: Any) -> bool:
+    """ Boolean indication of whether `val` is a valid `hdl21.Module` attribute. """
+    return isinstance(val, get_args(ModuleAttr))
+
+
+def _assert_module_attr(m: Module, val: Any) -> None:
+    """ Raise a TypeError if `val` is not a valid Module attribute. """
+    if not _is_module_attr(val):
+        _attr_type_error(m, val)
+
+
+def _attr_type_error(m: Module, val: Any) -> None:
+    """ Raise a `TypeError` with debug info for invalid attribute `val`. """
+
+    # Give more specific error-messages for our common types.
+    # Especially those that are easily confused, e.g. all the `Call` types, `Module`s, and `Instance`s thereof.
+    from .generator import Generator, GeneratorCall
+    from .primitives import Primitive, PrimitiveCall
+
+    if isinstance(val, (Generator, Primitive, ExternalModule)):
+        msg = f"Cannot add `{type(val).__name__}` `{val.name}` to `Module` `{m.name}`. Did you mean to make an `Instance` by *calling* it - once for params and once for connections - first?"
+    elif isinstance(val, Module):
+        msg = f"Cannot add `{type(val).__name__}` `{val.name}` to `Module` `{m.name}`. Did you mean to make an `Instance` by *calling* to connect it first?"
+    elif isinstance(val, (GeneratorCall, PrimitiveCall, ExternalModuleCall)):
+        msg = f"Cannot add `{type(val).__name__}` `{val}` to `Module` `{m.name}`. Did you mean to make an `Instance` by *calling* to connect it first?"
+    else:
+        msg = f"Invalid Module attribute {val} of type {type(val)} for {m}. Valid `Module` attributes are of types: {list(get_args(ModuleAttr))}"
+    raise TypeError(msg)
+
+
+def _caller_pymodule() -> Optional[ModuleType]:
+    """ # Caller Py-Module
+    Find the (python) module of the first python-stack-frame not from *this* file. 
+    
+    Sometimes this will be `hdl21.generator`. That's OK, `Generator`s know how to figure it out. 
+
+    Returns `None` if *no* python modules are in the call-stack. 
+    This can happen, for example when running `python -c "import hdl21 as h; h.Module()"`, 
+    or (probably) through other interactive interpreter setups. 
+
+    These "(python) module-less Modules" will generally fail in later steps 
+    such as exporting to ProtoBuf and netlists, but can be created in-memory. 
+    """
     for fr in inspect.stack():
         # Note frames produce an `Optional[ModuleType]`.
         # (It seems extension modules may not have one.)
@@ -407,4 +464,5 @@ def _caller_pymodule():
         pymod = inspect.getmodule(fr[0])
         if pymod is not None and pymod.__file__ != __file__:
             return pymod
-    raise RuntimeError("Could not find caller module")
+    # No caller module found, anywhere in the call-stack.
+    return None
